@@ -9,16 +9,14 @@
     :license: BSD, see LICENSE for details
 """
 from __future__ import absolute_import, division, print_function, with_statement
-
-from .formats import PANE_FORMATS
-
-from . import log, util
+from . import util, formats
 import logging
 
 logger = logging.getLogger(__name__)
 
 
 class Pane(util.TmuxMappingObject, util.TmuxRelationalObject):
+
     '''
         ``tmux(1)`` pane.
 
@@ -42,24 +40,38 @@ class Pane(util.TmuxMappingObject, util.TmuxRelationalObject):
         self.session = self.window.session
         self.server = self.session.server
 
-        self._TMUX = {}
-        self.update(**kwargs)
+        self._pane_id = kwargs['pane_id']
+
+        self.server._update_panes()
+        # self.update(**kwargs)
+
+    @property
+    def _TMUX(self, *args):
+
+        attrs = {
+            'pane_id': self._pane_id
+        }
+
+        # from https://github.com/serkanyersen/underscore.py
+        def by(val, *args):
+            for key, value in attrs.items():
+                try:
+                    if attrs[key] != val[key]:
+                        return False
+                except KeyError:
+                    return False
+                return True
+
+        return list(filter(by, self.server._panes))[0]
+
+    # def __getitem__(self, key):
+    #    return
 
     def tmux(self, *args, **kwargs):
-        #if '-t' not in kwargs:
+        # if '-t' not in kwargs:
         #    kwargs['-t'] = self.get['session_id']
         return self.server.tmux(*args, **kwargs)
 
-    def refresh(self):
-        '''Refresh current :class:`Pane` object. Chainable.
-
-        :rtype: :class:`Pane`
-        '''
-
-        self._TMUX = self.window.getById(self['pane_id'])._TMUX
-
-        return self
-
     def send_keys(self, cmd, enter=True):
         '''
             ```tmux send-keys``` to the pane
@@ -80,18 +92,24 @@ class Pane(util.TmuxMappingObject, util.TmuxRelationalObject):
         :rtype: :class:`Pane`
 
         '''
-        #if isinstance(target_pane, basestring) and not ':' not in target_pane or isinstance(target_pane, int):
+        # if isinstance(target_pane, basestring) and not ':' not in target_pane or isinstance(target_pane, int):
         #    target_pane = "%s.%s" % (self.target, target_pane)
 
-        #logger.error('resize-pane', '-t%s' % self.target)
+        # logger.error('resize-pane', '-t%s' % self.target)
         if 'height' in kwargs:
-            proc = self.tmux('resize-pane', '-t%s' % self.target, '-y%s' % int(kwargs['height']))
+            proc = self.tmux('resize-pane', '-t%s' %
+                             self.target, '-y%s' % int(kwargs['height']))
         elif 'width' in kwargs:
-            proc = self.tmux('resize-pane', '-t%s' % self.target, '-x%s' % int(kwargs['width']))
+            proc = self.tmux('resize-pane', '-t%s' %
+                             self.target, '-x%s' % int(kwargs['width']))
         else:
             proc = self.tmux('resize-pane', '-t%s' % self.target, args[0])
 
-        return self.refresh()
+        if proc.stderr:
+            raise Exception(proc.stderr)
+
+        self.server._update_panes()
+        return self
 
     def enter(self):
         '''
@@ -101,7 +119,8 @@ class Pane(util.TmuxMappingObject, util.TmuxRelationalObject):
 
     @property
     def target(self):
-        #return "%s:%s.%s" % (self.session.get('session_id'), self.get('window_id'), self.get('pane_index'))
+        # return "%s:%s.%s" % (self.session.get('session_id'),
+        # self.get('window_id'), self.get('pane_index'))
         return self.get('pane_id')
 
     def __repr__(self):
